#!/usr/bin/perl
#
# modem.pl - Report the status a Motorola 'SURFboard' cable modem
# (models: SB4200, SB5100, SB5100E, SB5101, SBV5120E)
#
# copyright(C) 2007 Scott Mazur <scott.AT.littlefish.ca>

# GNU General Public License
# --------------------------------------------------------------------
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA  02111-1307  USA

# requires:
#     Time::Local;
#     Socket;
#     RRDs;  (optional)

my $ver = '0.15';

# Version 0.15
# - confirmed model SB4200

# The Motorola 'SURFboard' cable modem includes a built in web
# interface that contains useful information like signal to noise
# ratios and power levels.
#
# This script uses LWP:Simple to scrape the status page from the
# modem and report the most useful information in a single line:
#   Down power, Signal to Noise, Up power
#
# If the script is not running interactively (like say from a cron
# job) the output line is prefixed with a standard log time stamp
# and Errors/warnings are duplicated to STDOUT and STDERR.  This
# is so that cron jobs can report errors while still sending
# output to a log file
#
# If the RRDs module is available, results are also posted
# to an RRDs database.

use strict;
use warnings;

use Time::Local;
use Socket;

# if the RRDs module is available, this is the path to the database
use constant RRDs_PATH => '/var/lib/RRDs';

# Reasonable ranges for signal levels
use constant MAX_DN_POWER => 12;
use constant MIN_DN_POWER => -12;
use constant MAX_UP_POWER => 54;
use constant MIN_UP_POWER => 36;
use constant MAX_SN_RATIO => 100; #FIXME reasonable value?
use constant MIN_SN_RATIO => 0;   #FIXME reasonable value?

use constant MODEM_IP => '192.168.100.1';
use constant SB5100_PATH => '/signaldata.html';
use constant SB5101_PATH => '/RgSignal.asp';
use constant SBV5120E_PATH => '/cmSignalData.htm?loginUsername=admin&loginPassword=motorola';

# are we running from a shell or maybe crond?
use constant I_am_interactive => (-t STDIN and -t STDOUT);

# format current time into standard system log file format
my $tm_str = localtime;
$tm_str = sprintf("$1 % 2d $3", $2)
	if $tm_str =~ m/^\S+\s+(\S+)\s+(\S+)\s+(\S+)/;

# print message in addition to die unless interactive
# This lets you pipe STDOUT to a file yet still get
# a message on the console (or from cron)
sub My_Die {
	my $msg = shift;
	print "$tm_str $msg\n" unless I_am_interactive;
	die "$msg\n";
}

sub Check_Range {
	my ($value, $max, $min, $desc) = @_;

	# strip down to raw number
	my ($raw_value) = $value =~ m/^([\d.-]+)/;

	my $status = '';
	$status = $desc . ' high!' if $raw_value > $max;
	$status = $desc . ' low!' if $raw_value < $min;

	# better call the cable guy!
	warn "$status $value\n" if $status;

	return $raw_value, $status;
}

# connect to the modem and retrieve the page in $path
sub Get_Page {
   my $path = shift;

	# open a tcp socket to the modem
	socket(my $socket, PF_INET, SOCK_STREAM, getprotobyname('tcp'));

	# connect with timeout
	my $failed = 1;
	eval {
		# set a signal to die if the timeout is reached
		local $SIG{ALRM} = sub { die "alarm\n" };
		# modem response should be quick!
		alarm 1; # 1 seconds
		connect($socket, sockaddr_in(80, inet_aton(MODEM_IP)))
			or My_Die("Couldn't connect to " . MODEM_IP . ":80 : $!");
		alarm 0;
		$failed = 0;
	};
	alarm 0; # prevent race condition
	My_Die("Couldn't connect to " . MODEM_IP . ":80 : Socket Timeout") if $failed;

	# enable command buffering (autoflush)
	select((select($socket), $| = 1)[0]);

	# send the page request
	print $socket join("\015\012",
						"GET $path HTTP/1.0",
						"Host: " . MODEM_IP,
						"User-Agent: modem.pl/$ver",
						"", "");

	# slurp the page results
	local $/;
	undef $/;
	my $buf = <$socket>;
	close($socket);

	return undef unless $buf;

	if ($buf =~ m,^HTTP/\d+\.\d+\s+(\d+)[^\012]*\012,) {
		my $code = $1;
		# print "CODE=$code\n$buf\n";
		# we don't handle redirects
		return undef unless $code =~ /^2/;
		$buf =~ s/.+?\015?\012\015?\012//s;  # zap header
	}

	return $buf;
}

# main script
#############

# get the modem status page
my $content = Get_Page(SB5100_PATH)
	|| Get_Page(SB5101_PATH)
	|| Get_Page(SBV5120E_PATH)
	|| My_Die("Couldn't get modem signal page!");

# The SBV5120E uses a login page.
# Because of this the page request sent to the modem
# is completely ignored and the first response returned
# is the login page.  A second page request is required
# to complete the login and get the signal page, after
# which the modem will remain 'logged in' for some period
# of time.
if ($content =~ m/loginUsername/i) {
	$content = Get_Page(SBV5120E_PATH)
	|| My_Die("Couldn't get modem signal page!");
}


# clean up the html a bit for parsing
$content =~ s/\n//g;     # drop new lines
$content =~ s!</?t[dr]>! !ig; # strip table tags
$content =~ s/&nbsp;/ /ig; # replace hard spaces
$content =~ s/\s\s+/ /g; # reduce double spaces

# check that the page has what we expect
$content =~ m{
	Frequency \s (\d+\s Hz) \s (Locked\s)?
	Signal \s To \s Noise \s Ratio \s ([\d.]+\s dB) \s
	.*?  # non-greedy extra stuff for SBV5120E
	Power \s Level \s ([\d.-]+\s dBmV)
	}xi or My_Die("Content failed!");

# build a hash of signal strength values
my %results = (
	DownStream => {freq => $1, sn => $3, power => $4},
	UpStream => {channel => '?', freq => '? Hz', power => '0  dBmV'}
	);

# get the upstream values
$results{UpStream} = {channel => $1, freq => $2, power => $5}
	if $content =~ m{
			ID \s (\d+) \s
			Frequency \s (\d+\s Hz)(\s Ranged)? \s
			.*?  # non-greedy extra stuff for SBV5120E
			Power(\s Level)?\s ([\d.-]+\s dBmV)
		}xi;

# create the result message
my $msg = "Down Power: $results{DownStream}->{power}" .
		"  S/N: $results{DownStream}->{sn}" .
		"  Up Power: $results{UpStream}->{power}";

# check up power for reasonable range
my ($raw_up_power, $status) =
	Check_Range($results{UpStream}->{power}, MAX_UP_POWER, MIN_UP_POWER, 'Up Power');
$msg .= " warning: $status" if $status;

# check down power for reasonable range
(my $raw_dn_power, $status) =
	Check_Range($results{DownStream}->{power}, MAX_DN_POWER, MIN_DN_POWER, 'Down Power');
$msg .= " warning: $status" if $status;

# check SN for reasonable range
(my $raw_SN, $status) =
	Check_Range($results{DownStream}->{sn}, MAX_SN_RATIO, MIN_SN_RATIO, 'SN Ratio');
$msg .= " warning: $status" if $status;

$msg = "$tm_str $msg" unless I_am_interactive;
print "$msg\n";

# rddtool stuff
###############
# the RRDs package may or may not be available
# so this section of code is wrapped in an eval
# just in case the package require fails
eval {
	require RRDs;

	my $RRDs_database = RRDs_PATH . '/modem.rrd';
	my $error;

	# does the modem rdd database exist?
	if (!RRDs::info($RRDs_database)) {
		warn "Create RRD database $RRDs_database\n";
		if (! -e RRDs_PATH) {
			mkdir(RRDs_PATH) or My_Die("Failed to created RRD databse directory " . RRDs_PATH . ": $!");
		}
		RRDs::create(
			$RRDs_database,
			"--step" , 300,
			"DS:up_power:GAUGE:600:10:80",
			"DS:down_power:GAUGE:600:-20:20",
			"DS:SN_ratio:GAUGE:600:0:100",
			"RRA:AVERAGE:0.5:1:576", # 48 hours at 5 minute intervals
			"RRA:AVERAGE:0.5:3:1344", # 2 weeks at 15 minute intervals
			"RRA:AVERAGE:0.5:72:240", # 2 months (60 days) at 6 hour intervals
			"RRA:AVERAGE:0.5:144:365"); # 1 year at 1 day intervals
		$error = RRDs::error();
		My_Die("ERROR while creating $RRDs_database: $error") if $error;
	}

	# insert values into rrd
	RRDs::update(
		$RRDs_database,
		"-t", "up_power:down_power:SN_ratio",
		"N:$raw_up_power:$raw_dn_power:$raw_SN");

	$error = RRDs::error();
	My_Die("ERROR while updating modem.rrd: $error") if $error;
};

# debug dump all values
# foreach (qw(DownStream UpStream)) {
# 	print "$_ - ";
# 	my $hash = $results{$_};
# 	foreach (sort keys %$hash) {
# 		print "  $_: $hash->{$_}";
# 	}
# 	print "\n";
# }

exit;
