#!/usr/bin/perl
#
# modem_graph.pl - Create modem status graphs
#
# copyright(C) 2007 Scott Mazur <scott.AT.littlefish.ca>

# GNU General Public License
# --------------------------------------------------------------------
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA  02111-1307  USA

# requires:
#     Time::Local;
#     RRDs;

my $ver = '0.02';

# Version 0.02
#  - Supress warnings from RRDs::graph

# This script creates 9 png image files using rrd_tools from the
# data created by modem.pl
# For each of Up Power, Down Power and SN Ratio, a graph is created
# for hourly, daily and monthly averages.

use strict;
use warnings;

use Time::Local;
use RRDs;

use constant RRDs_DATABASE => '/var/lib/RRDs/modem.rrd';

# change this if you need the images to go somewhere else
use constant IMAGE_PATH => '/var/www/html/images';

# Reasonable ranges for signal levels
use constant MAX_DN_POWER => 12;
use constant MIN_DN_POWER => -12;
use constant MAX_UP_POWER => 54;
use constant MIN_UP_POWER => 36;
use constant MAX_SN_RATIO => 100; #FIXME reasonable value?
use constant MIN_SN_RATIO => 0;   #FIXME reasonable value?

# are we running from a shell or maybe crond?
use constant I_am_interactive => (-t STDIN and -t STDOUT);

# print message in addition to die unless interactive
# This lets you pipe STDOUT to a file yet still get
# a message on the console (or from cron)
sub My_Die {
	my $msg = shift;
	# format current time into standard system log file format

	if (!I_am_interactive) {
		my $tm_str = localtime;
		$tm_str = sprintf("$1 % 2d $3", $2)
			if $tm_str =~ m/^\S+\s+(\S+)\s+(\S+)\s+(\S+)/;
		print "$tm_str $msg\n";
	}
	die "$msg\n";
}

# main script
#############

# file prefix, RRD data name, description, value type, *lower limit, *upper limit
#  *optional
my @graphs = (
		['modem-up', 'up_power', 'Up Stream Power Level', 'dBmV'], # MIN_UP_POWER, MAX_UP_POWER],
		['modem-down', 'down_power', 'Down Stream Power Level', 'dBmV'], # MIN_DN_POWER, MAX_DN_POWER],
		['modem-SN', 'SN_ratio', 'Signal To Noise Ratio', 'dB'] #, MIN_SN_RATIO, MAX_SN_RATIO]
	);

my $error;

foreach my $interval (qw(hour day week month)) {
	map {
		my $file = "$_->[0]_$interval.png";
		my $desc = $_->[1];
		# depending on your version of rrdtools and librsvg
		# this function may produce a harmless warning:
		# "attempt to put segment in horiz list twice"
		# for this reason STDERR is supressed during the graph
		# function and restored after.
		open(my $olderr, '>&', \*STDERR) or My_Die("Failed to copy STDERR: $!");
		open(STDERR, ">/dev/null"); # ignore STDERR
		RRDs::graph(
			IMAGE_PATH . "/$file",
			"--lazy",
			"-s -1$interval",
			"--title", "Motorola SURFboard: $_->[2] (1 $interval)",
			"--height", "200", "--width", "600",
			"--imgformat", "PNG",
			"--vertical-label", $_->[3],
			# limit Y range
			($_->[5] ? ("--lower-limit", $_->[4], "--upper-limit", $_->[5]) : ()),
			"--units-exponent", "0",  # don't scale Y range

			"DEF:value=" . RRDs_DATABASE . ":$_->[1]:AVERAGE",

			"LINE2:value#FF0000",

			"GPRINT:value:MIN:  Min\\: %2.1lf",
			"GPRINT:value:MAX: Max\\: %2.1lf",
			"GPRINT:value:AVERAGE: Avg\\: %4.1lf",
			"GPRINT:value:LAST: Current\\: %2.1lf $_->[3]\\n",
		);
		open(STDERR, '>&', $olderr) or My_Die("Failed to restore STDERR: $!");
		My_Die("ERROR unable to generate $file graph: $error")
			if $error = RRDs::error;
	} @graphs;
}

exit;
